"""
 Locations
"""
import json


class Location:

    def __init__(self, id, capacity, parent_id):
        self.id = id
        self.parent_id = parent_id
        self.children = []
        self.capacity = capacity
        self.sum_capacity = 0

    def decrease_capacity(self, delta):
        self.capacity -= delta
        return self.capacity


class Tree:

    def __init__(self):
        """
        Init for Tree
        """
        self.root = None
        self.ids = []

    def add_node(self, capacity, id, parent_id=None):

        if not self.root:
            self.root = Location(id, capacity, None)
            self.ids.append(id)
            return json.dumps({"ok": True})

        if parent_id not in self.ids:
            return json.dumps({"ok": False})

        res = self.__add_node(self.root, capacity, id, parent_id)
        if res == 0:
            return json.dumps({"ok": True})
        return json.dumps({"ok": False})

    def __add_node(self, node, capacity, id, parent_id):

        if node.id == parent_id:

            node.children.append(Location(id, capacity, parent_id))
            self.ids.append(id)
            return 0

        for child in node.children:
            res = self.__add_node(child, capacity, id, parent_id)
            if res in (0, -1):
                return res
        return 1

    def find_node(self, id):
        return self.__find_node(self.root, id)

    def __find_node(self, node, id):
        if node.id == id:
            return node
        for child in node.children:
            res = self.__find_node(child, id)
            if res:
                return res
        return None

    def generate_whole_path(self, type = 1):
        if type == 1:
            return self.__dfs(self.root, [])
        else:
            return self.__dfs2(self.root, [])

    def calculate_capacity(self, node):
        for child in sorted(node.children, key=lambda x: x.capacity, reverse=True):
            node.sum_capacity += self.calculate_capacity(child)
        if len(node.children) != 0:
            return node.sum_capacity
        else:
            return node.capacity

    def __dfs(self, node, sort_ids):
        for child in sorted(node.children, key=lambda x: x.capacity, reverse=True):
            sort_ids = self.__dfs(child, sort_ids)

        sort_ids.append(node.id)
        return sort_ids

    def __dfs2(self, node, sort_ids):
        for child in sorted(node.children, key=lambda x: x.capacity, reverse=True):
            sort_ids = self.__dfs2(child, sort_ids)
            sort_ids.append(node.id)

        sort_ids.append(node.id)
        return sort_ids


class Node:

    def __init__(self, id, name, parent=None):
        """
        Params:
           - name      {string}: Node name
           - id        {string}: Node ID
           - parent_id {string}: ID of the parent node; if ommitted or empty string,
                                 add this node as the root node (assuming there isn't
                                 already a root node)

        Creates list for the tree
        """
        self.id = id
        self.name = name
        self.children = []
        self.parent = parent

    def get_structure(self):
        """
        :return: structure for query
        """
        return {
            "id": self.id,
            "name": self.name,
            "parent_id": self.parent
        }

