from unittest import TestCase

from algorithm.core.Problem import VehicleRoutingProblem
from algorithm.core.Vehicle import Vehicle
from algorithm.core.Location import Location
from algorithm.exceptions.CustomExceptions import DuplicateVehicle, DuplicateLocation


class TestVehicleRoutingProblem(TestCase):
    def test_add_duplicate_vehicle(self):
        vrp = VehicleRoutingProblem()
        vrp.add_vehicle(Vehicle(1))
        with self.assertRaises(DuplicateVehicle):
            vrp.add_vehicle(Vehicle(1))

    def test_add_vehicle(self):
        vrp = VehicleRoutingProblem()
        self.assertEqual("added", vrp.add_vehicle(Vehicle(1)))

    def test_add_duplicate_location(self):
        vrp = VehicleRoutingProblem()
        vrp.add_location(Location(1))
        with self.assertRaises(DuplicateLocation):
            vrp.add_location(Location(1))

    def test_add_location(self):
        vrp = VehicleRoutingProblem()
        self.assertEqual("added", vrp.add_location(Location(1)))

    def test_solve(self):
        vrp = VehicleRoutingProblem()
        vrp.add_location(Location(0, 0, None))
        vrp.add_location(Location(1, 0, 0))
        vrp.add_location(Location(2, 0, 0))
        vrp.add_location(Location(3, 0, 1))
        vrp.add_location(Location(4, 0, 2))
        self.assertEqual([3, 1, 4, 2, 0], vrp.solve())

    def test_solve2(self):
        vrp = VehicleRoutingProblem()
        vrp.add_location(Location(0, 0, None))
        vrp.add_location(Location(1, 0, 0))
        vrp.add_location(Location(2, 0, 0))
        vrp.add_location(Location(3, 10, 1))
        vrp.add_location(Location(4, 10, 2))
        vrp.add_location(Location(5, 20, 1))
        vrp.add_location(Location(6, 5, 2))
        self.assertEqual([5, 3, 1, 4, 6, 2, 0], vrp.solve())

    def test_solve3(self):
        vrp = VehicleRoutingProblem()
        vrp.add_vehicle(Vehicle(1, 10))
        vrp.add_vehicle(Vehicle(2, 20))
        vrp.add_vehicle(Vehicle(3, 100))
        vrp.add_location(Location(0, 0, None))
        vrp.add_location(Location(1, 0, 0))
        vrp.add_location(Location(2, 0, 0))
        vrp.add_location(Location(3, 10, 1))
        vrp.add_location(Location(4, 10, 2))
        vrp.add_location(Location(5, 20, 1))
        vrp.add_location(Location(6, 5, 2))
        self.assertEqual([5, 3, 1, 4, 6, 2, 0], vrp.solve())